package restx.apidocs;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.ObjectWriter;
import com.google.common.base.Optional;
import static com.google.common.base.Preconditions.checkNotNull;

import restx.*;
import restx.factory.*;
import restx.description.*;
import restx.converters.MainStringConverter;
import static restx.common.MorePreconditions.checkPresent;

import javax.validation.Validator;
import static restx.validation.Validations.checkValid;

import java.io.IOException;
import java.io.PrintWriter;

@Component
public class SpecsResourceRouter extends RestxRouter {

    public SpecsResourceRouter(final SpecsResource resource, final ObjectMapper mapper, final MainStringConverter converter, final Validator validator) {
        super(
                "SpecsResourceRouter", new RestxRoute[] {
        new StdEntityRoute("admin#SpecsResource#findSpecsForOperation", mapper, new StdRouteMatcher("GET", "/@/specs")) {
            @Override
            protected Optional<?> doRoute(RestxRequest request, RestxRouteMatch match) throws IOException {
                return Optional.of(resource.findSpecsForOperation(
                        /* [QUERY] httpMethod */ checkPresent(request.getQueryParam("httpMethod"), "query param httpMethod is required"),
                        /* [QUERY] path */ checkPresent(request.getQueryParam("path"), "query param path is required")
                ));
            }

            protected ObjectWriter getObjectWriter(ObjectMapper mapper) { return super.getObjectWriter(mapper).withType(new TypeReference<java.lang.Iterable<java.lang.String>>() { }); }

            @Override
            protected void describeOperation(OperationDescription operation) {
                                OperationParameterDescription httpMethod = new OperationParameterDescription();
                httpMethod.name = "httpMethod";
                httpMethod.paramType = OperationParameterDescription.ParamType.query;
                httpMethod.dataType = "string";
                httpMethod.required = false;
                operation.parameters.add(httpMethod);

                OperationParameterDescription path = new OperationParameterDescription();
                path.name = "path";
                path.paramType = OperationParameterDescription.ParamType.query;
                path.dataType = "string";
                path.required = false;
                operation.parameters.add(path);


                operation.responseClass = "LIST[string]";
            }
        },
        new StdEntityRoute("admin#SpecsResource#getSpecById", mapper, new StdRouteMatcher("GET", "/@/specs/{id}")) {
            @Override
            protected Optional<?> doRoute(RestxRequest request, RestxRouteMatch match) throws IOException {
                return resource.getSpecById(
                        /* [PATH] id */ checkNotNull(match.getPathParams().get("id"), "path param id is required")
                );
            }

            

            @Override
            protected void describeOperation(OperationDescription operation) {
                                OperationParameterDescription id = new OperationParameterDescription();
                id.name = "id";
                id.paramType = OperationParameterDescription.ParamType.path;
                id.dataType = "string";
                id.required = false;
                operation.parameters.add(id);


                operation.responseClass = "RestxSpec";
            }
        },
        });
    }

}
